library benchpress.src.web_driver_extension_dart;

import 'package:angular2/di.dart' show bind, Injector, OpaqueToken;
import 'package:angular2/src/facade/lang.dart'
    show BaseException, ABSTRACT, isBlank;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, Map;
import './sample_options.dart' show Options;

@ABSTRACT()
class WebDriverExtension {
  static bindTo(childTokens) {
    return [
      bind(_CHILDREN).toAsyncFactory((injector) {
        return PromiseWrapper.all(ListWrapper.map(childTokens, (token) {
          return injector.asyncGet(token);
        }));
      }, [Injector]),
      bind(WebDriverExtension).toFactory((children, capabilities) {
        var delegate;
        ListWrapper.forEach(children, (extension) {
          if (extension.supports(capabilities)) {
            delegate = extension;
          }
        });
        if (isBlank(delegate)) {
          throw new BaseException(
              'Could not find a delegate for given capabilities!');
        }
        return delegate;
      }, [_CHILDREN, Options.CAPABILITIES])
    ];
  }
  Future gc() {
    throw new BaseException('NYI');
  }
  Future timeBegin(name) {
    throw new BaseException('NYI');
  }
  Future timeEnd(name, bool restart) {
    throw new BaseException('NYI');
  }
  Future<List> readPerfLog() {
    throw new BaseException('NYI');
  }
  bool supports(Map capabilities) {
    return true;
  }
}
var _CHILDREN = new OpaqueToken('WebDriverExtension.children');
