library benchpress.test.reporter.multi_reporter_spec_dart;

import 'package:angular2/test_lib.dart'
    show ddescribe, describe, it, iit, xit, expect, beforeEach, afterEach;
import 'package:angular2/src/facade/collection.dart'
    show List, ListWrapper, Map;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper, Future;
import 'package:angular2/src/facade/lang.dart' show DateWrapper;
import 'package:benchpress/benchpress.dart'
    show Reporter, MultiReporter, bind, Injector, MeasureValues;

main() {
  createReporters(ids) {
    return new Injector([
      ListWrapper.map(ids, (id) {
        return bind(id).toValue(new MockReporter(id));
      }),
      MultiReporter.createBindings(ids)
    ]).asyncGet(MultiReporter);
  }
  describe('multi reporter', () {
    it('should reportMeasureValues to all', (done) {
      var mv = new MeasureValues(0, DateWrapper.now(), {});
      createReporters(['m1', 'm2']).then((r) {
        return r.reportMeasureValues(mv);
      }).then((values) {
        expect(values)
            .toEqual([{'id': 'm1', 'values': mv}, {'id': 'm2', 'values': mv}]);
        done();
      });
    });
    it('should reportSample to call', (done) {
      var completeSample = [
        new MeasureValues(0, DateWrapper.now(), {}),
        new MeasureValues(1, DateWrapper.now(), {})
      ];
      var validSample = [completeSample[1]];
      createReporters(['m1', 'm2']).then((r) {
        return r.reportSample(completeSample, validSample);
      }).then((values) {
        expect(values).toEqual([
          {
            'id': 'm1',
            'completeSample': completeSample,
            'validSample': validSample
          },
          {
            'id': 'm2',
            'completeSample': completeSample,
            'validSample': validSample
          }
        ]);
        done();
      });
    });
  });
}
class MockReporter extends Reporter {
  String _id;
  MockReporter(id) : super() {
    this._id = id;
  }
  Future reportMeasureValues(MeasureValues values) {
    return PromiseWrapper.resolve({'id': this._id, 'values': values});
  }
  Future reportSample(
      List<MeasureValues> completeSample, List<MeasureValues> validSample) {
    return PromiseWrapper.resolve({
      'id': this._id,
      'completeSample': completeSample,
      'validSample': validSample
    });
  }
}
