library benchpress.test.runner_spec_dart;

import 'package:angular2/test_lib.dart'
    show describe, it, iit, xit, expect, beforeEach, afterEach;
import 'package:benchpress/benchpress.dart'
    show
        Runner,
        Sampler,
        SampleDescription,
        Validator,
        bind,
        Injector,
        Metric,
        Options,
        WebDriverAdapter;
import 'package:angular2/src/facade/lang.dart' show isBlank;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;

main() {
  describe('runner', () {
    var injector;
    var runner;
    createRunner([defaultBindings = null]) {
      if (isBlank(defaultBindings)) {
        defaultBindings = [];
      }
      runner = new Runner([
        defaultBindings,
        bind(Sampler).toFactory((_injector) {
          injector = _injector;
          return new MockSampler();
        }, [Injector]),
        bind(Metric).toFactory(() {
          return new MockMetric();
        }, []),
        bind(Validator).toFactory(() {
          return new MockValidator();
        }, []),
        bind(WebDriverAdapter).toFactory(() {
          return new MockWebDriverAdapter();
        }, [])
      ]);
      return runner;
    }
    it('should set SampleDescription.id', (done) {
      createRunner().sample(id: 'someId').then((_) {
        return injector.asyncGet(SampleDescription);
      }).then((desc) {
        expect(desc.id).toBe('someId');
        done();
      });
    });
    it('should merge SampleDescription.description', (done) {
      createRunner([bind(Options.DEFAULT_DESCRIPTION).toValue({'a': 1})])
          .sample(
              id: 'someId',
              bindings: [bind(Options.SAMPLE_DESCRIPTION).toValue({'b': 2})])
          .then((_) {
        return injector.asyncGet(SampleDescription);
      }).then((desc) {
        expect(desc.description).toEqual({
          'forceGc': false,
          'userAgent': 'someUserAgent',
          'a': 1,
          'b': 2,
          'v': 11
        });
        done();
      });
    });
    it('should fill SampleDescription.metrics from the Metric', (done) {
      createRunner().sample(id: 'someId').then((_) {
        return injector.asyncGet(SampleDescription);
      }).then((desc) {
        expect(desc.metrics).toEqual({'m1': 'some metric'});
        done();
      });
    });
    it('should bind Options.EXECUTE', (done) {
      var execute = () {};
      createRunner().sample(id: 'someId', execute: execute).then((_) {
        expect(injector.get(Options.EXECUTE)).toEqual(execute);
        done();
      });
    });
    it('should bind Options.PREPARE', (done) {
      var prepare = () {};
      createRunner().sample(id: 'someId', prepare: prepare).then((_) {
        expect(injector.get(Options.PREPARE)).toEqual(prepare);
        done();
      });
    });
    it('should overwrite bindings per sample call', (done) {
      createRunner([bind(Options.DEFAULT_DESCRIPTION).toValue({'a': 1})])
          .sample(
              id: 'someId',
              bindings: [bind(Options.DEFAULT_DESCRIPTION).toValue({'a': 2})])
          .then((_) {
        return injector.asyncGet(SampleDescription);
      }).then((desc) {
        expect(injector.get(SampleDescription).description['a']).toBe(2);
        done();
      });
    });
  });
}
class MockWebDriverAdapter extends WebDriverAdapter {
  Future executeScript(script) {
    return PromiseWrapper.resolve('someUserAgent');
  }
}
class MockValidator extends Validator {
  MockValidator() : super() {}
  describe() {
    return {'v': 11};
  }
}
class MockMetric extends Metric {
  MockMetric() : super() {}
  describe() {
    return {'m1': 'some metric'};
  }
}
class MockSampler extends Sampler {
  MockSampler() : super() {}
  Future sample() {
    return PromiseWrapper.resolve(23);
  }
}
